@@ -2,21 +2,24 @@
  * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -266,6 +269,12 @@ void IOFireWireSBP2ManagementORB::free( void )
         fStatusBlockAddressSpace->release();
     }
 
+	if( fResponseMap != NULL )
+	{
+		fResponseMap->release();
+		fResponseMap = NULL;
+	}
+	
     //
     // deallocate response address space
     //
@@ -392,8 +401,8 @@ IOReturn IOFireWireSBP2ManagementORB::setResponseBuffer
 								( IOMemoryDescriptor * desc )
 {
     IOReturn				status 			= kIOReturnSuccess;
-    void * 					buf;
-    UInt32 					len;
+    void * 					buf = NULL;
+    UInt32 					len = 0;
     
     //
     // deallocate old response address space
@@ -405,20 +414,22 @@ IOReturn IOFireWireSBP2ManagementORB::setResponseBuffer
         fResponseAddressSpace->release();
     }
 
-    if( desc != NULL )
+	if( fResponseMap != NULL )
+	{
+		fResponseMap->release();
+	}
+    
+	if( desc != NULL )
     {
         len = desc->getLength();
-  //  buf = desc->getBytesNoCopy();
+		
+		fResponseMap = desc->map();
+		if( fResponseMap != NULL )
+		{
+			buf = (void*)fResponseMap->getVirtualAddress();
+		}
+     }
   
-	IOByteCount length = 0;
-	buf = desc->getVirtualSegment( 0, &length );
-     }
-     else
-     {
-	len = 0;
-	buf = NULL;
-     }
-
 	fResponseBuf = buf;
 	fResponseLen = len;
 
